/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
    Author: Marcin O'BenY Benka
    email: obeny@obeny.kicks-ass.net
    gg uin: 473891
    jabber: obeny@chrome.pl
*/
		
// private header
#include "contacts.h"
		    
// kadu headers
#include "chatbutton.h"
#include "contacts_dialog.h"
#include "gadu.h"
#include "debug.h"
#include "misc.h"
#include "modules.h"

using namespace std;

Contacts* contacts;

/*
    Initialization and Cleanup
*/
extern "C" int contacts_init()
{
	contacts = new Contacts();

	return 0;
}

extern "C" void contacts_close()
{
	delete contacts;
}

/*
    Constructor and destructor
*/
Contacts::Contacts() : QObject(NULL, "contacts")
{
	kdebugf();
	
        // when new chat is created, handle it
        connect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
        connect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));

        // handle created chat windows
        ChatList::ConstIterator it;
        for(it = chat_manager->chats().begin(); it != chat_manager->chats().end(); ++it)
	        handleCreatedChat(*it);

	buttonPressed = false;

	kdebugf2();
}

Contacts::~Contacts()
{
	kdebugf();

        // when chat is destroyed, handle it
        disconnect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
        disconnect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));
    
        // handle chat windows that are being destroyed
        ChatList::ConstIterator it;
        for(it = chat_manager->chats().begin(); it != chat_manager->chats().end(); ++it)
	        handleDestroyingChat(*it);

	kdebugf2();
}

void Contacts::chatCreated(const UinsList& ul)
{
        kdebugf();
	
	Chat* chat = chat_manager->findChatByUins(ul);
	handleCreatedChat(chat);
		
	kdebugf2();
}

void Contacts::chatDestroying(const UinsList& ul)
{
	kdebugf();
	
	Chat* chat = chat_manager->findChatByUins(ul);
	handleDestroyingChat(chat);
	
	kdebugf2();
}

void Contacts::handleCreatedChat(Chat* chat)
{
        kdebugf();
	
        ChatButton* cs = new ChatButton(chat);
        cs->setPixmap(icons_manager.loadIcon(dataPath("kadu/modules/data/contacts/contacts_small.png")));
        cs->show();
        QToolTip::add(cs, tr("Send Contact(s)"));
        chatButtons[chat] = cs;
        connect(cs, SIGNAL(clicked(Chat*)), this, SLOT(chooseContacts(Chat*)));
	
	kdebugf2();
}

void Contacts::handleDestroyingChat(Chat* chat)
{
        kdebugf();

	disconnect(chatButtons[chat], SIGNAL(clicked(Chat*)), this, SLOT(chooseContacts(Chat*)));
	delete chatButtons[chat];
	chatButtons.remove(chat);
	
	kdebugf2();
}

void Contacts::chooseContacts(Chat* chat)
{
	kdebugf();

	QDialog* choose_contacts_dlg = new QDialog(chat);

	choose_contacts_dlg->exec();

	delete choose_contacts_dlg;
	
	kdebugf2();
}
