/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
    Author: Marcin O'BenY Benka
    email: obeny@obeny.kicks-ass.net
    gg uin: 473891
    jabber: obeny@chrome.pl
*/

/*
    part of code is taken from spy, iwait4u and xmms module (it saved lot of my time)
*/

// private header
#include "shellexec.h"

// taken from PStreams Library (version 0.5.2) avalaible at http://pstreams.sourceforge.net
#include "pstream.h"

// kadu headers
#include "config_dialog.h"
#include "config_file.h"
#include "gadu.h"
#include "debug.h"
#include "misc.h"

using namespace std;

Shellexec* shellexec;

/*
    Initialization and Cleanup
*/
extern "C" int shellexec_init()
{
	shellexec = new Shellexec();

	// connect to chat message receive event
	QObject::connect(gadu, SIGNAL(chatMsgReceived1(UinsList, const QString&, time_t, bool&)),
	    shellexec, SLOT(sendRemoteCmdOutput(UinsList, const QString&, time_t)));

	return 0;
}

extern "C" void shellexec_close()
{
	// disconnect from chat message receive event
	QObject::disconnect(gadu, SIGNAL(chatMsgReceived1(UinsList, const QString&, time_t, bool&)),
    	    shellexec, SLOT(sendRemoteCmdOutput(UinsList, const QString&, time_t)));

	delete shellexec;
}

/*
    Constructor and destructor
*/
Shellexec::Shellexec() : QObject(NULL, "shellexec")
{
	kdebugf();

	// when new chat is created, handle it
	connect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
	connect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));

	// handle created chat windows
	ChatList::ConstIterator it;
	for (it = chat_manager->chats().begin(); it != chat_manager->chats().end(); ++it)
		handleCreatedChat(*it);

	// get my uin for further removal from list in configuration
	MyUin = config_file.readNumEntry("General", "UIN");

	// load trusted commands and users to memory
	loadTrustedCommandsList();
	loadTrustedUsersList();

	// create configuration GUI
	createConfGui();

	kdebugf2();
}

Shellexec::~Shellexec()
{
	kdebugf();

	// when chat is destroyed, handle it
	disconnect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
	disconnect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));

	// handle chat windows that are being destroyed
	ChatList::ConstIterator it;
	for (it = chat_manager->chats().begin(); it != chat_manager->chats().end(); ++it)
		handleDestroyingChat(*it);

	// save trusted commands and users to disk
	saveTrustedCommandsList();
	saveTrustedUsersList();

	// destroy configuration GUI
	destroyConfGui();
}

/********************
    MEMBERS
********************/

// check whether command is trusted
bool Shellexec::isTrustedCommand(QString command)
{
	kdebugf();

        for (unsigned int i = 0; i < trustedCommands.count(); ++i)
	{
		if(!(trustedCommands[i].compare(command)))
		{
	    		kdebugf2();
			return true;
		}
	}

	kdebugf2();

	return false;
}

// check whether user is trusted
bool Shellexec::isTrustedUser(UinType uin)
{
	kdebugf();

	vector<UinType>::iterator it = find(trustedUsers.begin(), trustedUsers.end(), uin);
	if (it != trustedUsers.end())
	{
		kdebugf2();
		return true;
	}

	kdebugf2();

	return false;
}

// create configuration dialog GUI
void Shellexec::createConfGui()
{
	kdebugf();

	// creating configuration tab
	ConfigDialog::addTab(QT_TRANSLATE_NOOP("&default", "Shellexec"), dataPath("kadu/modules/data/shellexec/shellexec.png"));

	// adding checkboxes
	ConfigDialog::addCheckBox("Shellexec", "Shellexec",
	    QT_TRANSLATE_NOOP("&default", "Allow remote shell commands execution"),
	    "RemoteExec", config_file.readBoolEntry("Shellexec", "RemoteExec"));

	ConfigDialog::addCheckBox("Shellexec", "Shellexec",
	    QT_TRANSLATE_NOOP("&default", "Allow trusted users executing all commands"),
	    "RemoteExecTrustedUsersAllCmds", config_file.readBoolEntry("Shellexec", "RemoteExecTrustedUsersAllCmds"));

	// drawing trusted users selection
	ConfigDialog::addGrid("Shellexec", "Shellexec", "shellexec_listboxes", 3);
	ConfigDialog::addGrid("Shellexec", "shellexec_listboxes", "shellexec_listbox1", 1);
	ConfigDialog::addLabel("Shellexec", "shellexec_listbox1", QT_TRANSLATE_NOOP("@default", "Available"));
	ConfigDialog::addListBox("Shellexec", "shellexec_listbox1", "shellexec_available");
	ConfigDialog::addGrid("Shellexec", "shellexec_listboxes", "shellexec_listbox2", 1);

	ConfigDialog::addPushButton("Shellexec", "shellexec_listbox2", "", "AddToNotifyList", "", "shellexec_forward");
	ConfigDialog::addPushButton("Shellexec", "shellexec_listbox2", "", "RemoveFromNotifyList", "", "shellexec_back");

	ConfigDialog::addGrid("Shellexec", "shellexec_listboxes", "shellexec_listbox3", 1);
	ConfigDialog::addLabel("Shellexec", "shellexec_listbox3", QT_TRANSLATE_NOOP("@default", "Trusted users"));
	ConfigDialog::addListBox("Shellexec", "shellexec_listbox3", "shellexec_trustedUsers");

	// adding trusted commands list
	ConfigDialog::addTextEdit("Shellexec", "Shellexec", QT_TRANSLATE_NOOP("@default", "Trusted commands"), "TrustedCommands", "", "List of trusted commands");

	// connecting slots to signals
	ConfigDialog::connectSlot("Shellexec", "", SIGNAL(clicked()),
	    this, SLOT(_Right()), "shellexec_forward");
	ConfigDialog::connectSlot("Shellexec", "", SIGNAL(clicked()),
	    this, SLOT(_Left()), "shellexec_back");
	ConfigDialog::connectSlot("Shellexec", "shellexec_available", SIGNAL(doubleClicked(QListBoxItem*)),
	    this, SLOT(_Right2(QListBoxItem*)));
	ConfigDialog::connectSlot("Shellexec", "shellexec_trustedUsers", SIGNAL(doubleClicked(QListBoxItem*)),
	    this, SLOT(_Left2(QListBoxItem*)));

	ConfigDialog::connectSlot("Shellexec", "Allow remote shell commands execution", SIGNAL(toggled(bool)),
	    this, SLOT(onRemoteExec(bool)));
	ConfigDialog::connectSlot("Shellexec", "Allow trusted users executing all commands", SIGNAL(toggled(bool)),
	    this, SLOT(onRemoteExecTrustedUsersAllCmds(bool)));

	// reacting on creation of config dialog and applying changes
	ConfigDialog::registerSlotOnCreate(this, SLOT(onCreateConfigDialog()));
	ConfigDialog::registerSlotOnApply(this, SLOT(onApplyConfigDialog()));

	kdebugf2();
}

// destroy configuration dialog GUI
void Shellexec::destroyConfGui()
{
	kdebugf();

	// disconnecting slots from signals
	ConfigDialog::disconnectSlot("Shellexec", "", SIGNAL(clicked()),
	    this, SLOT(_Right()), "shellexec_forward");
	ConfigDialog::disconnectSlot("Shellexec", "", SIGNAL(clicked()),
	    this, SLOT(_Left()), "shellexec_back");
	ConfigDialog::disconnectSlot("Shellexec", "shellexec_available", SIGNAL(doubleClicked(QListBoxItem*)),
	    this, SLOT(_Right2(QListBoxItem*)));
	ConfigDialog::disconnectSlot("Shellexec", "shellexec_trustedUsers", SIGNAL(doubleClicked(QListBoxItem*)),
	    this, SLOT(_Left2(QListBoxItem*)));

	ConfigDialog::disconnectSlot("Shellexec", "Allow remote shell commands execution", SIGNAL(toggled(bool)),
	    this, SLOT(onRemoteExec(bool)));
	ConfigDialog::disconnectSlot("Shellexec", "Allow trusted users executing all commands", SIGNAL(toggled(bool)),
	    this, SLOT(onRemoteExecTrustedUsersAllCmds(bool)));

	// releasing reacting on creation of config dialog and applying changes
	ConfigDialog::unregisterSlotOnCreate(this, SLOT(onCreateConfigDialog()));
	ConfigDialog::unregisterSlotOnApply(this, SLOT(onApplyConfigDialog()));

	// removing trusted commands list
	ConfigDialog::removeControl("Shellexec", "Trusted commands");

	// removing trusted users selection
	ConfigDialog::removeControl("Shellexec", "shellexec_trustedUsers");
	ConfigDialog::removeControl("Shellexec", "Trusted users");
	ConfigDialog::removeControl("Shellexec", "shellexec_listbox3");

	ConfigDialog::removeControl("Shellexec", "", "shellexec_back");
	ConfigDialog::removeControl("Shellexec", "", "shellexec_forward");

	ConfigDialog::removeControl("Shellexec", "shellexec_listbox2");
	ConfigDialog::removeControl("Shellexec", "shellexec_available");
	ConfigDialog::removeControl("Shellexec", "Available");
	ConfigDialog::removeControl("Shellexec", "shellexec_listbox1");
	ConfigDialog::removeControl("Shellexec", "shellexec_listboxes");

	// removing checkboxes
	ConfigDialog::removeControl("Shellexec", "Allow remote shell commands execution");

	ConfigDialog::removeControl("Shellexec", "Allow trusted users executing all commands");

	// destroying configuration tab
	ConfigDialog::removeTab("Shellexec");

	kdebugf2();
}

// execute command and put it's result in chat window
void Shellexec::executeCommand(Chat* chat)
{
	kdebugf();
	
	redi::ipstream result(command.ascii());

	string outputLine;

	while(getline(result, outputLine))
		resp += QString(outputLine + "\n");

	chat->edit->insertLine(QString::fromLocal8Bit(resp));

	kdebugf2();
}

// connect message sending to checking whether it isn't shell command
void Shellexec::handleCreatedChat(Chat* chat)
{
	kdebugf();

	connect(chat, SIGNAL(messageSendRequested(Chat*)), this, SLOT(sendCmdOutput(Chat*)));

	kdebugf2();
}

// disconnect message sending from checking whether it isn't shell command						
void Shellexec::handleDestroyingChat(Chat* chat)
{
	kdebugf();

	disconnect(chat, SIGNAL(messageSendRequested(Chat*)), this, SLOT(sendCmdOutput(Chat*)));

	kdebugf2();
}

// load trusted commands list from config file
void Shellexec::loadTrustedCommandsList()
{
	kdebugf();

	trustedCommands = QStringList::split('\n', config_file.readEntry("Shellexec", "TrustedCommands", ""));

	kdebugf2();
}

// save trusted commands list to config file
void Shellexec::saveTrustedCommandsList()
{
	kdebugf();
	
	config_file.writeEntry("Shellexec", "TrustedCommands", trustedCommands.join("\n"));

	kdebugf2();
}

// load trusted users list from file
void Shellexec::loadTrustedUsersList()
{
	kdebugf();

	ifstream ifs(ggPath("trusted-users").ascii());
	if(!ifs.is_open())
	{
		kdebugf2();
		return;
	}

	while (!ifs.eof())
	{
		UinType uin;
		ifs >> uin;
		if(ifs)
			trustedUsers.push_back(uin);
	}

	ifs.close();

	kdebugf2();
}

// save trusted users list to file
void Shellexec::saveTrustedUsersList()
{
	kdebugf();

	ofstream ofs(ggPath("trusted-users").ascii());
	if(!ofs.is_open())
	{
	        kdebugf2();
	        return;
	}

	for (unsigned int i = 0; i < trustedUsers.size(); i++)
		ofs << trustedUsers[i] << '\n';

	ofs.close();

	kdebugf2();
}

// adding trusted command to list
void Shellexec::trustedCommandAdd(QString command)
{
	kdebugf();

	if(!(isTrustedCommand(command)))
	{
		trustedCommands += command;

		// it has to be written to disk
		saveTrustedCommandsList();
	}

	kdebugf2();
}

// adding trusted user to list
void Shellexec::trustedUserAdd(UinType uin)
{
	kdebugf();
	
	vector<UinType>::iterator it = find(trustedUsers.begin(), trustedUsers.end(), uin);
	if(it == trustedUsers.end())
		trustedUsers.push_back(uin);
	
	kdebugf2();
}

// removing trusted user from list
void Shellexec::trustedUserRemove(UinType uin)
{
	kdebugf();

	vector<UinType>::iterator it = find(trustedUsers.begin(), trustedUsers.end(), uin);
	if(it != trustedUsers.end())
		trustedUsers.erase(it);

	kdebugf2();
}

// END
