/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// private header
#include "shellexec.h"

// taken from PStreams Library (version 0.5.2) avalaible at http://pstreams.sourceforge.net
#include "pstream.h"

// kadu headers
#include "config_dialog.h"
#include "config_file.h"
#include "gadu.h"
#include "debug.h"
#include "misc.h"

// QT headers
#include <qcheckbox.h>
#include <qgrid.h>
#include <qmessagebox.h>

using namespace std;

/********************
    PUBLIC SLOTS
********************/

// sets enabled or not elements in config dialog on "Allow remote command execution"
void Shellexec::onRemoteExec(bool toggled)
{
	kdebugf();

	ConfigDialog::getGrid("Shellexec", "shellexec_listboxes")->setEnabled(toggled);
	ConfigDialog::getTextEdit("Shellexec", "Trusted commands")->setEnabled(toggled);
	ConfigDialog::getCheckBox("Shellexec", "Allow trusted users executing all commands")->setEnabled(toggled);

	kdebugf2();
}

// sets enabled or not trusted command list in config dialog on "Allow trusted users executing all commands"
void Shellexec::onRemoteExecTrustedUsersAllCmds(bool toggled)
{
	kdebugf();

	ConfigDialog::getTextEdit("Shellexec", "Trusted commands")->setEnabled(!toggled);
	
	kdebugf2();
}

// check whether my message isn't shell command execution request. If is, execute command and send output
void Shellexec::sendCmdOutput(Chat* chat)
{
	kdebugf();

	QString msg = chat->edit->textLine(0);

	if(msg.left(7) == "/shell ")
	{
		command = msg.right(msg.length() - 7);
		resp = QString("\nKADU SHELLEXEC:\n\n");

		executeCommand(chat);
	}

	kdebugf2();
}

// check whether received message isn't shell command execution request
void Shellexec::sendRemoteCmdOutput(UinsList sender, const QString& msg, time_t time)
{
	kdebugf();

	if(config_file.readBoolEntry("Shellexec", "RemoteExec"))
	{
		Chat* chat = chat_manager->findChatByUins(sender);
		
		// don't accept remoteshell executions if chat is a conference
		if(msg.left(13) == "/remoteshell " && sender.count() == 1)
		{
			int SenderUin = userlist.byUinValue(chat->uins()[0]).uin();
			command = msg.right(msg.length() - 13);

			string outputLine;

			if(isTrustedUser(SenderUin) && 
			    (config_file.readBoolEntry("Shellexec", "RemoteExecTrustedUsersAllCmds") || isTrustedCommand(command)))
	    		{
				redi::ipstream result(command.ascii());

				resp = msg + QString("\n\nKADU REMOTE SHELLEXEC:\n\n");

				while(getline(result, outputLine))
					resp += QString(outputLine + "\n");

				gadu->sendMessage(sender, QString::fromLocal8Bit(resp));
			}
			else
			{
                                switch(QMessageBox::question(NULL, tr("Remote command execution request"),
				tr("User %1 requested execution of following command:"
				"\n\n\t%2\n\n\tWould you like to proceed this command?")
				    .arg(userlist.byUinValue(chat->uins()[0]).altNick())
				    .arg(command.ascii()),
				tr("&No"), tr("&Yes"), tr("Yes and add to trusted list"), 0, 1))
				{
					// rejected
					case 0:
						gadu->sendMessage(sender, tr("Command cannot be executed!"));
						break;
					// accepted
					case 1:
						resp = msg + QString("\n\nKADU REMOTE SHELLEXEC:\n\n");
						executeCommand(chat);
						break;
					// accpeted with adding to trusted commands list
					case 2:
						redi::ipstream result(command.ascii());

						resp = msg + QString("\n\nKADU REMOTE SHELLEXEC:\n\n");

						while(getline(result, outputLine))
							resp += QString(outputLine + "\n");

						gadu->sendMessage(sender, QString::fromLocal8Bit(resp));

						trustedCommandAdd(command);
						break;
				}
			}

		}
	}

	kdebugf2();
}

// removing user from trused users list on icon click
void Shellexec::_Left()
{
	kdebugf();

	QListBox *e_availableusers = ConfigDialog::getListBox("Shellexec", "shellexec_available");
	QListBox *e_trustedusers = ConfigDialog::getListBox("Shellexec", "shellexec_trustedUsers");

	QStringList tomove;

	unsigned int i;
	
	for(i = 0; i < e_trustedusers->count(); ++i)
		if(e_trustedusers->isSelected(i))

	tomove += e_trustedusers->text(i);

	for(i = 0; i < tomove.size(); ++i)
	{
		e_availableusers->insertItem(tomove[i]);
		e_trustedusers->removeItem(e_trustedusers->index(e_trustedusers->findItem(tomove[i])));
	}
	e_availableusers->sort();

	kdebugf2();
}

// adding user to trused users list on icon click
void Shellexec::_Right()
{
	kdebugf();

	QListBox *e_availableusers = ConfigDialog::getListBox("Shellexec", "shellexec_available");
	QListBox *e_trustedusers = ConfigDialog::getListBox("Shellexec", "shellexec_trustedUsers");

	QStringList tomove;

	unsigned int i;

	for(i = 0; i < e_availableusers->count(); ++i)
		if(e_availableusers->isSelected(i))

	tomove += e_availableusers->text(i);

	for(i = 0; i < tomove.size(); ++i)
	{
		e_trustedusers->insertItem(tomove[i]);
		e_availableusers->removeItem(e_availableusers->index(e_availableusers->findItem(tomove[i])));
	}
	e_trustedusers->sort();

	kdebugf2();
}

// removing user from trused users list on user doubleclick
void Shellexec::_Left2(QListBoxItem*)
{
	_Left();
}

// adding user to trused users list on user doubleclick
void Shellexec::_Right2(QListBoxItem*)
{
	_Right();
}

/********************
    PRIVATE SLOTS
********************/

// slot is executed when chat window is created
void Shellexec::chatCreated(const UinsList& ul)
{
	kdebugf();

	Chat* chat = chat_manager->findChatByUins(ul);
		handleCreatedChat(chat);

	kdebugf2();
}

// slot is executed when chat window is destroyed
void Shellexec::chatDestroying(const UinsList& ul)
{
	kdebugf();

	Chat* chat = chat_manager->findChatByUins(ul);
		handleDestroyingChat(chat);

	kdebugf2();
}

// slot is executed on applying changes to config dialog
void Shellexec::onApplyConfigDialog()
{
	kdebugf();

	vector<UinType> oldTrustedUsers(trustedUsers);
	
	QListBox *e_trustedusers = ConfigDialog::getListBox("Shellexec", "shellexec_trustedUsers");

	trustedUsers.clear();

	for (unsigned int i = 0; i < e_trustedusers->count(); ++i)
	{
		int uin = userlist.byAltNick(e_trustedusers->text(i)).uin();
		trustedUserAdd(uin);
	}

	saveTrustedUsersList();

	// config file is saved, so we can reload trused commands list
	loadTrustedCommandsList();

	kdebugf2();
}

// slot is executed on creating config dialog
void Shellexec::onCreateConfigDialog()
{
	QListBox *e_availableusers = ConfigDialog::getListBox("Shellexec", "shellexec_available");
	QListBox *e_trustedusers = ConfigDialog::getListBox("Shellexec", "shellexec_trustedUsers");

	for(UserList::ConstIterator i = userlist.begin(); i != userlist.end(); ++i)
	{
		int uin = (*i).uin();
		if(!(uin == MyUin) && (*i).uin())
		{
			if(isTrustedUser((*i).uin()))
				e_trustedusers->insertItem((*i).altNick());
			else
				e_availableusers->insertItem((*i).altNick());
		}
	}
        e_availableusers->sort();
        e_trustedusers->sort();
        e_availableusers->setSelectionMode(QListBox::Extended);
        e_trustedusers->setSelectionMode(QListBox::Extended);

	QCheckBox* b_remoteexec = ConfigDialog::getCheckBox("Shellexec", "Allow remote shell commands execution");
	QCheckBox* b_remoteexectrustedusersallcmds = ConfigDialog::getCheckBox("Shellexec", "Allow trusted users executing all commands");
	QTextEdit* t_trustedusers = ConfigDialog::getTextEdit("Shellexec", "Trusted commands");
	
	if(b_remoteexec->isChecked())
	{
		t_trustedusers->setEnabled(!(b_remoteexectrustedusersallcmds->isChecked()));
	}
	else
	{
		ConfigDialog::getGrid("Shellexec", "shellexec_listboxes")->setEnabled(false);
		b_remoteexectrustedusersallcmds->setEnabled(false);
		t_trustedusers->setEnabled(false);
	}
}

// END
